Package: interSAE
Type: Package
Title: Intersectional Small Area Estimation from Survey and Census Data
Version: 0.1.0
Authors@R: person(
    given = "Leila",
    family = "Marvian Mashhad",
    email = "leila.marveian@gmail.com",
    role = c("aut", "cre")
    )
Description: Implements design-aware, margin-constrained estimation of
    population indicators for geographic areas crossed with demographic
    subgroups. Complex-survey microdata supply outcomes and association
    information, while one or more aggregate census tables supply overlapping
    population margins. A generalized iterative proportional fitting engine
    reconstructs a coherent latent population table, and an augmented
    model-assisted estimator produces domain means, proportions, and totals
    for sampled and unsampled intersections. Tools
    diagnose non-identification, compute linear-programming sensitivity bounds,
    propagate sampling and model uncertainty with replicate-weight or
    multiplier bootstrap procedures, enforce structural zeros, and benchmark
    estimates to official totals. The framework extends calibration ideas from
    Deville and Sarndal (1992) <doi:10.1080/01621459.1992.10475217> and small
    area estimation ideas from Fay and Herriot (1979)
    <doi:10.1080/01621459.1979.10482505>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: graphics, stats, utils
Suggests: knitr, lpSolve, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-08-24 16:38:42 UTC; marvian
Author: Leila Marvian Mashhad [aut, cre]
Maintainer: Leila Marvian Mashhad <leila.marveian@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-25 05:20:31 UTC

export(benchmark_inter)
export(census_margins)
export(check_identifiability)
export(decompose_uncertainty)
export(fit_inter_sae)
export(identification_bounds)
export(inter_bootstrap)
export(inter_design)
export(inter_estimates)
export(inter_example)
export(inter_sensitivity)
export(margin_diagnostics)
export(simulate_inter_data)
S3method(plot,inter_sae)
S3method(print,census_margins)
S3method(print,inter_bootstrap)
S3method(print,inter_bounds)
S3method(print,inter_design)
S3method(print,inter_identifiability)
S3method(print,inter_sae)
S3method(print,summary_inter_sae)
S3method(summary,inter_sae)

# interSAE

`interSAE` implements joint margin-constrained intersectional small area
estimation from complex-survey microdata and aggregate census margins.

The motivating problem is common in official statistics: a survey observes an
outcome and several demographic variables, while published census tables give
precise but overlapping margins rather than a full confidential microdata file.
Direct survey estimates become unstable or unavailable after crossing a small
geographic area with age, sex, residence, or another subgroup.

## What the package does

1. Validates survey weights, strata, primary sampling units, and optional
   replicate weights.
2. Validates overlapping census tables and structural-zero declarations.
3. Reconstructs a coherent latent joint population table with generalized
   iterative proportional fitting.
4. Fits a survey-weighted Gaussian or binomial outcome model.
5. Produces augmented, model-assisted means, proportions, and totals.
6. Diagnoses the rank and nullity of the margin system.
7. Optionally computes linear-programming identification bounds and gamma
   sensitivity envelopes.
8. Refits the entire procedure under multiplier or supplied replicate weights.
9. Benchmarks domain estimates to an official aggregate.

## Installation

```r
install.packages("interSAE_0.1.0.tar.gz", repos = NULL, type = "source")
```

The core package uses only R recommended packages. Install `lpSolve` to compute
identification bounds.

## Minimal example

```r
library(interSAE)

x <- inter_example()

design <- inter_design(
  x$survey,
  weights = "weight",
  area = "area",
  strata = "strata",
  psu = "psu"
)

margins <- census_margins(x$margins)

fit <- fit_inter_sae(
  unemployed ~ sex + age + residence + area,
  design = design,
  margins = margins,
  family = "binomial",
  domain = c("area", "sex")
)

inter_estimates(fit)
margin_diagnostics(fit)
check_identifiability(fit)
```

Optional sensitivity analysis:

```r
if (requireNamespace("lpSolve", quietly = TRUE)) {
  inter_sensitivity(
    fit,
    gamma = c(1, 1.25, 1.5, 2, Inf),
    domain = list(area = "A1", sex = "Female")
  )
}
```

## Interpretation

The fitted point estimate depends on the outcome model and on a coherent
population reconstruction seeded by the weighted survey association structure.
When aggregate margins do not identify the full joint table, the package says
so explicitly. Identification bounds vary the latent joint table while holding
published census margins fixed. Bootstrap intervals and identification bounds
answer different questions and should be reported separately.

## Scope of version 0.1.0

This release is a research implementation for categorical census dimensions
and Gaussian or binary survey outcomes. It does not claim that aggregate
margins alone identify a confidential joint population distribution. Applied
work should document the outcome formula, structural zeros, sensitivity values,
survey design, and benchmarking decisions.

#' Create a design-aware survey object
#'
#' @param data Survey microdata.
#' @param weights,strata,psu,area Column names supplied as strings.
#' @param replicate_weights Optional character vector naming replicate-weight
#'   columns.
#' @return An object of class `inter_design`.
#' @export
inter_design <- function(data, weights, area, strata = NULL, psu = NULL,
                         replicate_weights = NULL) {
  .inter_assert_data_frame(data, "data")
  weights <- .inter_assert_column(data, weights, "weights")
  area <- .inter_assert_column(data, area, "area")
  strata <- .inter_assert_column(data, strata, "strata", allow_null = TRUE)
  psu <- .inter_assert_column(data, psu, "psu", allow_null = TRUE)

  if (!is.null(replicate_weights)) {
    if (!is.character(replicate_weights) || !length(replicate_weights) ||
        !all(replicate_weights %in% names(data))) {
      stop("`replicate_weights` must name columns in `data`.", call. = FALSE)
    }
  }
  w <- data[[weights]]
  if (!is.numeric(w) || any(!is.finite(w)) || any(w < 0) || sum(w) <= 0) {
    stop("Survey weights must be finite, non-negative, and have a positive sum.",
         call. = FALSE)
  }
  if (anyNA(data[[area]])) {
    stop("The area variable may not contain missing values.", call. = FALSE)
  }

  structure(
    list(
      data = data,
      weights = weights,
      area = area,
      strata = strata,
      psu = psu,
      replicate_weights = replicate_weights
    ),
    class = "inter_design"
  )
}

#' @export
print.inter_design <- function(x, ...) {
  cat("Intersectional survey design\n")
  cat("  Records: ", nrow(x$data), "\n", sep = "")
  cat("  Area: ", x$area, "\n", sep = "")
  cat("  Weight: ", x$weights, "\n", sep = "")
  if (!is.null(x$strata)) cat("  Strata: ", x$strata, "\n", sep = "")
  if (!is.null(x$psu)) cat("  PSU: ", x$psu, "\n", sep = "")
  if (!is.null(x$replicate_weights)) {
    cat("  Replicate weights: ", length(x$replicate_weights), "\n", sep = "")
  }
  invisible(x)
}

test_that("identification bounds contain the fitted synthetic estimate", {
  skip_if_not_installed("lpSolve")
  x <- inter_example()
  fit <- fit_inter_sae(
    unemployed ~ sex + age + residence + area,
    inter_design(x$survey, "weight", "area"),
    census_margins(x$margins),
    family = "binomial"
  )
  bounds <- identification_bounds(fit, list(area = "A1"), gamma = 1.5)
  expect_s3_class(bounds, "inter_bounds")
  expect_lte(bounds$lower, bounds$estimate + 1e-8)
  expect_gte(bounds$upper, bounds$estimate - 1e-8)
})

test_that("a short multiplier bootstrap returns replicates", {
  x <- inter_example()
  fit <- fit_inter_sae(
    unemployed ~ sex + age + residence + area,
    inter_design(x$survey, "weight", "area", "strata", "psu"),
    census_margins(x$margins),
    family = "binomial"
  )
  boot <- inter_bootstrap(fit, R = 3, seed = 7)
  expect_s3_class(boot, "inter_bootstrap")
  expect_equal(ncol(boot$replicates), 3L)
})

#' Validate aggregate census margins
#'
#' @param margins A data frame or named list of aggregate census tables.
#' @param count Name of the population-count column in every table.
#' @param structural_zeros Optional data frame. Each row describes an impossible
#'   combination; `NA` acts as a wildcard.
#' @param tolerance Relative tolerance for comparing table totals.
#' @param reconcile If `TRUE`, rescale inconsistent table totals to their mean.
#' @return An object of class `census_margins`.
#' @export
census_margins <- function(margins, count = "Freq", structural_zeros = NULL,
                           tolerance = 1e-8, reconcile = FALSE) {
  if (is.data.frame(margins)) {
    margins <- list(margin1 = margins)
  }
  if (!is.list(margins) || !length(margins) ||
      !all(vapply(margins, is.data.frame, logical(1)))) {
    stop("`margins` must be a data frame or a non-empty list of data frames.",
         call. = FALSE)
  }
  if (is.null(names(margins)) || any(!nzchar(names(margins)))) {
    names(margins) <- paste0("margin", seq_along(margins))
  }
  if (!is.character(count) || length(count) != 1L) {
    stop("`count` must be one column name.", call. = FALSE)
  }

  cleaned <- vector("list", length(margins))
  names(cleaned) <- names(margins)
  totals <- numeric(length(margins))
  dimensions <- vector("list", length(margins))

  for (i in seq_along(margins)) {
    table <- margins[[i]]
    if (!count %in% names(table)) {
      stop(sprintf("Census table `%s` has no `%s` column.", names(margins)[i],
                   count), call. = FALSE)
    }
    dims <- setdiff(names(table), count)
    if (!length(dims)) {
      stop("Every census table must contain at least one dimension.",
           call. = FALSE)
    }
    values <- table[[count]]
    if (!is.numeric(values) || any(!is.finite(values)) || any(values < 0)) {
      stop("Census counts must be finite and non-negative.", call. = FALSE)
    }
    if (anyNA(table[dims])) {
      stop("Census dimensions may not contain missing values.", call. = FALSE)
    }
    key <- .inter_key(table, dims)
    if (anyDuplicated(key)) {
      table <- stats::aggregate(table[[count]], table[dims], sum,
                                na.rm = TRUE, drop = FALSE)
      names(table)[ncol(table)] <- count
    }
    cleaned[[i]] <- table
    totals[i] <- sum(table[[count]])
    dimensions[[i]] <- dims
  }

  target_total <- mean(totals)
  relative_difference <- max(abs(totals - target_total)) / max(1, target_total)
  if (relative_difference > tolerance) {
    if (!isTRUE(reconcile)) {
      stop(paste0(
        "Census margin totals are inconsistent. Use `reconcile = TRUE` only ",
        "if proportional rescaling is substantively justified."
      ),
           call. = FALSE)
    }
    for (i in seq_along(cleaned)) {
      if (totals[i] <= 0) {
        stop("A zero-total census table cannot be reconciled.", call. = FALSE)
      }
      cleaned[[i]][[count]] <- cleaned[[i]][[count]] * target_total / totals[i]
    }
    totals[] <- target_total
  }

  if (!is.null(structural_zeros)) {
    .inter_assert_data_frame(structural_zeros, "structural_zeros")
    all_dims <- unique(unlist(dimensions, use.names = FALSE))
    if (!all(names(structural_zeros) %in% all_dims)) {
      stop("Structural-zero columns must be census dimensions.", call. = FALSE)
    }
  }

  structure(
    list(
      tables = cleaned,
      count = count,
      dimensions = unique(unlist(dimensions, use.names = FALSE)),
      table_dimensions = dimensions,
      total = target_total,
      original_totals = totals,
      reconciled = relative_difference > tolerance,
      structural_zeros = structural_zeros,
      tolerance = tolerance
    ),
    class = "census_margins"
  )
}

#' @export
print.census_margins <- function(x, ...) {
  cat("Validated census margins\n")
  cat("  Tables: ", length(x$tables), "\n", sep = "")
  cat("  Dimensions: ", paste(x$dimensions, collapse = ", "), "\n", sep = "")
  cat("  Population total: ", format(x$total, scientific = FALSE), "\n", sep = "")
  cat("  Reconciled: ", if (x$reconciled) "yes" else "no", "\n", sep = "")
  invisible(x)
}

.inter_assert_data_frame <- function(x, arg) {
  if (!is.data.frame(x)) {
    stop(sprintf("`%s` must be a data frame.", arg), call. = FALSE)
  }
  invisible(TRUE)
}

.inter_assert_column <- function(data, column, arg, allow_null = FALSE) {
  if (is.null(column) && allow_null) {
    return(NULL)
  }
  if (!is.character(column) || length(column) != 1L || is.na(column)) {
    stop(sprintf("`%s` must be one column name supplied as a string.", arg),
         call. = FALSE)
  }
  if (!column %in% names(data)) {
    stop(sprintf("Column `%s`, supplied to `%s`, was not found.", column, arg),
         call. = FALSE)
  }
  column
}

.inter_key <- function(data, variables) {
  if (!length(variables)) {
    return(rep.int(".all", nrow(data)))
  }
  values <- lapply(data[variables], function(x) {
    x <- as.character(x)
    x[is.na(x)] <- "<NA>"
    gsub("\\u001f", "<US>", x, fixed = TRUE)
  })
  do.call(paste, c(values, sep = "\u001f"))
}

.inter_family <- function(family) {
  if (inherits(family, "family")) {
    return(family)
  }
  if (!is.character(family) || length(family) != 1L) {
    stop("`family` must be a family object or one of 'gaussian' and 'binomial'.",
         call. = FALSE)
  }
  switch(
    match.arg(family, c("gaussian", "binomial")),
    gaussian = stats::gaussian(),
    binomial = stats::binomial()
  )
}

.inter_weighted_mean <- function(x, w) {
  ok <- is.finite(x) & is.finite(w) & w > 0
  if (!any(ok)) {
    return(NA_real_)
  }
  sum(x[ok] * w[ok]) / sum(w[ok])
}

.inter_group_sum <- function(x, key, all_keys = NULL) {
  ans <- tapply(x, key, sum, na.rm = TRUE)
  if (is.null(all_keys)) {
    return(ans)
  }
  out <- stats::setNames(rep.int(0, length(all_keys)), all_keys)
  common <- intersect(names(ans), all_keys)
  out[common] <- ans[common]
  out
}

.inter_match_rows <- function(x, specification) {
  if (is.null(specification) || !length(specification)) {
    return(rep.int(TRUE, nrow(x)))
  }
  if (is.data.frame(specification)) {
    if (!nrow(specification)) {
      return(rep.int(FALSE, nrow(x)))
    }
    vars <- names(specification)
    if (!all(vars %in% names(x))) {
      stop("Structural-zero variables must occur in the census dimensions.",
           call. = FALSE)
    }
    hit <- rep.int(FALSE, nrow(x))
    for (i in seq_len(nrow(specification))) {
      current <- rep.int(TRUE, nrow(x))
      for (variable in vars) {
        value <- specification[[variable]][i]
        if (!is.na(value)) {
          current <- current & as.character(x[[variable]]) == as.character(value)
        }
      }
      hit <- hit | current
    }
    return(hit)
  }
  if (!is.list(specification) || is.null(names(specification))) {
    stop("A domain must be a named list or a data frame.", call. = FALSE)
  }
  if (!all(names(specification) %in% names(x))) {
    stop("Domain variables must occur in the reconstructed population table.",
         call. = FALSE)
  }
  hit <- rep.int(TRUE, nrow(x))
  for (variable in names(specification)) {
    hit <- hit & as.character(x[[variable]]) %in%
      as.character(specification[[variable]])
  }
  hit
}

.inter_aggregate <- function(data, variables, value, fun = sum) {
  if (!length(variables)) {
    out <- data.frame(.all = "All", stringsAsFactors = FALSE)
    out[[value]] <- fun(data[[value]])
    return(out)
  }
  stats::aggregate(data[[value]], data[variables], fun,
                   na.rm = TRUE, drop = FALSE)
}

.inter_restore_rng <- function(old_seed) {
  if (is.null(old_seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      rm(".Random.seed", envir = .GlobalEnv)
    }
  } else {
    assign(".Random.seed", old_seed, envir = .GlobalEnv)
  }
}

#' @export
print.inter_sae <- function(x, ...) {
  cat("Joint margin-constrained intersectional estimates\n")
  cat("  Formula: ", paste(deparse(x$formula), collapse = " "), "\n", sep = "")
  cat("  Family: ", x$family_label, "\n", sep = "")
  cat("  Domains: ", paste(x$domain, collapse = " x "), "\n", sep = "")
  cat("  Latent population cells: ", nrow(x$population$grid), "\n", sep = "")
  cat("  Margin convergence: ", if (x$population$converged) "yes" else "no",
      " (max relative error ", format(x$population$error, digits = 3), ")\n",
      sep = "")
  print(utils::head(inter_estimates(x), 10L), row.names = FALSE)
  invisible(x)
}

#' @export
summary.inter_sae <- function(object, ...) {
  identification <- check_identifiability(object)
  structure(
    list(
      call = object$call,
      formula = object$formula,
      family = object$family_label,
      survey_records = nrow(object$design$data),
      domains = object$domain,
      latent_cells = nrow(object$population$grid),
      population_total = sum(object$population$grid$.population),
      converged = object$population$converged,
      iterations = object$population$iterations,
      margin_error = object$population$error,
      nullity = identification$nullity,
      estimates = inter_estimates(object)
    ),
    class = "summary_inter_sae"
  )
}

#' @export
print.summary_inter_sae <- function(x, ...) {
  cat("Summary of interSAE fit\n")
  cat("  Survey records: ", x$survey_records, "\n", sep = "")
  cat("  Population total: ", format(x$population_total, scientific = FALSE),
      "\n", sep = "")
  cat("  Latent cells: ", x$latent_cells, "\n", sep = "")
  cat("  Unidentified dimensions: ", x$nullity, "\n", sep = "")
  cat("  Reconstruction iterations: ", x$iterations, "\n", sep = "")
  cat("  Maximum relative margin error: ",
      format(x$margin_error, digits = 4), "\n", sep = "")
  print(x$estimates, row.names = FALSE)
  invisible(x)
}

#' Plot intersectional estimates
#'
#' @param x A fitted `inter_sae` object.
#' @param y Ignored.
#' @param compare_direct Show direct survey estimates when available.
#' @param ... Additional arguments passed to [graphics::dotchart()].
#' @return The plotted data, invisibly.
#' @export
plot.inter_sae <- function(x, y = NULL, compare_direct = TRUE, ...) {
  estimates <- x$estimates
  label <- apply(estimates[x$domain], 1L, paste, collapse = " | ")
  order_index <- order(estimates$estimate)
  values <- estimates$estimate[order_index]
  labels <- label[order_index]
  graphics::dotchart(values, labels = labels,
                     xlab = if (x$family_label == "binomial")
                       "Estimated proportion" else "Estimated mean",
                     pch = 19, color = "#1B6CA8", ...)
  if (isTRUE(compare_direct)) {
    direct <- estimates$direct[order_index]
    ok <- is.finite(direct)
    graphics::points(direct[ok], which(ok), pch = 1, col = "#B23A48")
    graphics::legend("bottomright", legend = c("interSAE", "Direct"),
                     pch = c(19, 1), col = c("#1B6CA8", "#B23A48"),
                     bty = "n")
  }
  invisible(estimates[order_index, , drop = FALSE])
}

#' Simulate survey microdata and overlapping census margins
#'
#' @param seed Integer random-number seed.
#' @param n Survey sample size.
#' @param areas Character vector of area labels.
#' @return A list with `survey`, `margins`, and the latent `truth` table.
#' @export
simulate_inter_data <- function(seed = 123, n = 500,
                                areas = paste0("A", 1:6)) {
  if (!is.numeric(seed) || length(seed) != 1L || !is.finite(seed)) {
    stop("`seed` must be one finite number.", call. = FALSE)
  }
  if (!is.numeric(n) || length(n) != 1L || n < 50) {
    stop("`n` must be at least 50.", call. = FALSE)
  }
  if (!is.character(areas) || length(areas) < 2L || anyDuplicated(areas)) {
    stop("`areas` must contain at least two distinct labels.", call. = FALSE)
  }
  old_seed <- if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
    get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  } else {
    NULL
  }
  on.exit(.inter_restore_rng(old_seed), add = TRUE)
  set.seed(as.integer(seed))

  truth <- expand.grid(
    area = areas,
    sex = c("Female", "Male"),
    age = c("18-34", "35-54", "55+"),
    residence = c("Rural", "Urban"),
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  area_effect <- stats::setNames(seq(-0.45, 0.45, length.out = length(areas)),
                                areas)
  log_size <- 6.0 +
    0.20 * (truth$sex == "Female") +
    0.30 * (truth$residence == "Urban") +
    0.18 * (truth$age == "35-54") -
    0.12 * (truth$age == "55+") +
    area_effect[truth$area]
  truth$population <- pmax(25, round(exp(log_size)))
  linear_predictor <- -1.25 +
    0.35 * (truth$sex == "Female") +
    0.55 * (truth$age == "18-34") -
    0.30 * (truth$age == "55+") +
    0.45 * (truth$residence == "Rural") +
    0.25 * (truth$sex == "Female" & truth$residence == "Rural") +
    area_effect[truth$area]
  truth$probability <- stats::plogis(linear_predictor)

  chosen <- sample.int(nrow(truth), size = as.integer(n), replace = TRUE,
                       prob = truth$population)
  survey <- truth[chosen, c("area", "sex", "age", "residence"), drop = FALSE]
  survey$unemployed <- stats::rbinom(nrow(survey), 1,
                                     truth$probability[chosen])
  sample_counts <- table(chosen)
  survey$weight <- truth$population[chosen] /
    as.numeric(sample_counts[as.character(chosen)])
  survey$strata <- survey$area
  survey$psu <- paste0(survey$area, "-", sample.int(5L, nrow(survey),
                                                    replace = TRUE))
  rownames(survey) <- NULL

  aggregate_margin <- function(variables) {
    out <- stats::aggregate(truth$population, truth[variables], sum,
                            drop = FALSE)
    names(out)[ncol(out)] <- "Freq"
    out
  }
  margins <- list(
    area_sex_age = aggregate_margin(c("area", "sex", "age")),
    area_sex_residence = aggregate_margin(c("area", "sex", "residence")),
    area_age = aggregate_margin(c("area", "age"))
  )
  list(survey = survey, margins = margins, truth = truth)
}

#' Load the reproducible interSAE example
#'
#' @return The same components as [simulate_inter_data()] using fixed settings.
#' @export
inter_example <- function() {
  simulate_inter_data(seed = 2026, n = 480)
}

.inter_build_grid <- function(design, margins, max_cells = 1e6) {
  dims <- margins$dimensions
  missing_survey <- setdiff(dims, names(design$data))
  if (length(missing_survey)) {
    stop("All census dimensions must occur in the survey data. Missing: ",
         paste(missing_survey, collapse = ", "), call. = FALSE)
  }
  if (!design$area %in% dims) {
    stop("The survey area variable must occur in at least one census margin.",
         call. = FALSE)
  }

  levels_by_dimension <- lapply(dims, function(variable) {
    values <- as.character(design$data[[variable]])
    for (table in margins$tables) {
      if (variable %in% names(table)) {
        values <- c(values, as.character(table[[variable]]))
      }
    }
    sort(unique(values[!is.na(values)]))
  })
  names(levels_by_dimension) <- dims
  cell_count <- prod(vapply(levels_by_dimension, length, integer(1)))
  if (!is.finite(cell_count) || cell_count > max_cells) {
    stop("The implied joint census table is too large. Collapse categories or increase `max_cells` deliberately.",
         call. = FALSE)
  }
  grid <- expand.grid(levels_by_dimension, KEEP.OUT.ATTRS = FALSE,
                      stringsAsFactors = FALSE)
  if (!is.null(margins$structural_zeros) && nrow(margins$structural_zeros)) {
    zero <- .inter_match_rows(grid, margins$structural_zeros)
    grid <- grid[!zero, , drop = FALSE]
  }
  if (!nrow(grid)) {
    stop("Structural-zero rules removed every population cell.", call. = FALSE)
  }
  rownames(grid) <- NULL
  grid$.cell <- seq_len(nrow(grid))
  grid
}

.inter_seed_counts <- function(grid, design, dims, total, prior) {
  if (!is.numeric(prior) || length(prior) != 1L || !is.finite(prior) ||
      prior <= 0) {
    stop("`prior` must be one positive finite number.", call. = FALSE)
  }
  grid_key <- .inter_key(grid, dims)
  survey_key <- .inter_key(design$data, dims)
  mapped <- match(survey_key, grid_key)
  positive <- design$data[[design$weights]] > 0
  if (any(is.na(mapped) & positive)) {
    stop("Positive-weight survey records conflict with structural-zero rules or census levels.",
         call. = FALSE)
  }
  observed <- numeric(nrow(grid))
  observed_by_cell <- tapply(
    design$data[[design$weights]][!is.na(mapped)],
    mapped[!is.na(mapped)],
    sum,
    na.rm = TRUE
  )
  if (length(observed_by_cell)) {
    observed[as.integer(names(observed_by_cell))] <-
      as.numeric(observed_by_cell)
  }
  seed <- observed + prior
  seed * total / sum(seed)
}

.inter_margin_maps <- function(grid, margins) {
  maps <- vector("list", length(margins$tables))
  names(maps) <- names(margins$tables)
  for (i in seq_along(margins$tables)) {
    table <- margins$tables[[i]]
    dims <- margins$table_dimensions[[i]]
    grid_key <- .inter_key(grid, dims)
    groups <- unique(grid_key)
    table_key <- .inter_key(table, dims)
    target <- stats::setNames(table[[margins$count]], table_key)
    target_for_group <- as.numeric(target[groups])
    target_for_group[is.na(target_for_group)] <- 0
    maps[[i]] <- list(
      name = names(margins$tables)[i],
      dimensions = dims,
      group = match(grid_key, groups),
      groups = groups,
      target = target_for_group
    )
  }
  maps
}

.inter_reconstruct <- function(design, margins, prior = 0.5,
                               max_iter = 1000L, tol = 1e-8,
                               max_cells = 1e6) {
  grid <- .inter_build_grid(design, margins, max_cells = max_cells)
  counts <- .inter_seed_counts(grid, design, margins$dimensions,
                               margins$total, prior)
  seed <- counts
  maps <- .inter_margin_maps(grid, margins)
  converged <- FALSE
  error <- Inf

  for (iteration in seq_len(max_iter)) {
    for (map in maps) {
      current <- as.numeric(rowsum(counts, map$group, reorder = FALSE))
      impossible <- map$target > 0 & current <= 0
      if (any(impossible)) {
        stop("The census margins and structural-zero rules are incompatible.",
             call. = FALSE)
      }
      ratio <- rep.int(0, length(current))
      positive <- map$target > 0
      ratio[positive] <- map$target[positive] / current[positive]
      counts <- counts * ratio[map$group]
    }
    errors <- vapply(maps, function(map) {
      current <- as.numeric(rowsum(counts, map$group, reorder = FALSE))
      max(abs(current - map$target) / pmax(1, map$target))
    }, numeric(1))
    error <- max(errors)
    if (is.finite(error) && error <= tol) {
      converged <- TRUE
      break
    }
  }
  if (!converged) {
    warning("Population reconstruction did not reach the requested tolerance.",
            call. = FALSE)
  }
  grid$.population <- counts
  grid$.seed <- seed
  structure(
    list(
      grid = grid,
      maps = maps,
      iterations = iteration,
      error = error,
      converged = converged,
      tolerance = tol,
      prior = prior
    ),
    class = "inter_population"
  )
}

.inter_constraint_system <- function(grid, margins) {
  maps <- .inter_margin_maps(grid, margins)
  rows <- sum(vapply(maps, function(x) length(x$target), integer(1)))
  A <- matrix(0, nrow = rows, ncol = nrow(grid))
  b <- numeric(rows)
  labels <- character(rows)
  cursor <- 0L
  for (map in maps) {
    for (j in seq_along(map$target)) {
      cursor <- cursor + 1L
      A[cursor, map$group == j] <- 1
      b[cursor] <- map$target[j]
      labels[cursor] <- paste0(map$name, ":", map$groups[j])
    }
  }
  keep <- !duplicated(cbind(A, b))
  list(A = A[keep, , drop = FALSE], b = b[keep], labels = labels[keep])
}

#' Diagnose agreement with census margins
#'
#' @param object A fitted `inter_sae` object.
#' @return A data frame containing target, reconstructed value, error, and
#'   relative error for every published census cell.
#' @export
margin_diagnostics <- function(object) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  counts <- object$population$grid$.population
  pieces <- lapply(object$population$maps, function(map) {
    current <- as.numeric(rowsum(counts, map$group, reorder = FALSE))
    data.frame(
      table = map$name,
      cell = map$groups,
      target = map$target,
      reconstructed = current,
      error = current - map$target,
      relative_error = abs(current - map$target) / pmax(1, map$target),
      stringsAsFactors = FALSE
    )
  })
  do.call(rbind, pieces)
}

#' Diagnose identification of the latent census table
#'
#' @param object A fitted `inter_sae` object.
#' @param tol Numerical rank tolerance.
#' @return An object of class `inter_identifiability`.
#' @export
check_identifiability <- function(object, tol = 1e-9) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  system <- .inter_constraint_system(object$population$grid, object$margins)
  rank <- qr(system$A, tol = tol)$rank
  cells <- ncol(system$A)
  result <- structure(
    list(
      cells = cells,
      constraints = nrow(system$A),
      rank = rank,
      nullity = cells - rank,
      table_identified = cells == rank,
      structural_zeros = if (is.null(object$margins$structural_zeros)) 0L else
        nrow(object$margins$structural_zeros),
      tolerance = tol
    ),
    class = "inter_identifiability"
  )
  result
}

#' @export
print.inter_identifiability <- function(x, ...) {
  cat("Identification diagnostic\n")
  cat("  Feasible cells: ", x$cells, "\n", sep = "")
  cat("  Independent margin constraints: ", x$rank, "\n", sep = "")
  cat("  Unidentified dimensions: ", x$nullity, "\n", sep = "")
  cat("  Full table identified: ", if (x$table_identified) "yes" else "no",
      "\n", sep = "")
  invisible(x)
}

.inter_lp <- function(direction, objective, mat, dir, rhs) {
  solution <- lpSolve::lp(
    direction = direction,
    objective.in = objective,
    const.mat = mat,
    const.dir = dir,
    const.rhs = rhs,
    all.int = FALSE,
    all.bin = FALSE,
    transpose.constraints = TRUE
  )
  if (solution$status != 0L) {
    stop("The requested identification problem is infeasible or unbounded (lpSolve status ",
         solution$status, ").", call. = FALSE)
  }
  solution$objval
}

.inter_domain_label <- function(domain) {
  if (is.null(domain) || !length(domain)) return("All")
  paste(paste0(names(domain), "=", vapply(domain, function(x) {
    paste(as.character(x), collapse = "|")
  }, character(1))), collapse = ", ")
}

#' Compute census-margin identification bounds
#'
#' @param object A fitted `inter_sae` object.
#' @param domain Optional named list defining one target domain.
#' @param gamma Association sensitivity parameter. `Inf` uses only the census
#'   margins; finite values constrain each latent cell to lie between its fitted
#'   population divided by `gamma` and multiplied by `gamma`.
#' @param statistic Bound a domain `mean`/proportion or a predicted `total`.
#' @return An object of class `inter_bounds`.
#' @export
identification_bounds <- function(object, domain = NULL, gamma = Inf,
                                  statistic = c("mean", "total")) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  if (!requireNamespace("lpSolve", quietly = TRUE)) {
    stop("Package `lpSolve` is required for identification bounds.",
         call. = FALSE)
  }
  statistic <- match.arg(statistic)
  if (!is.numeric(gamma) || length(gamma) != 1L || is.na(gamma) || gamma < 1) {
    stop("`gamma` must be one number greater than or equal to 1, or `Inf`.",
         call. = FALSE)
  }
  grid <- object$population$grid
  mask <- .inter_match_rows(grid, domain)
  if (!any(mask)) {
    stop("The requested domain contains no feasible population cells.",
         call. = FALSE)
  }
  predicted <- grid$.prediction
  reference <- grid$.population
  p <- as.numeric(mask) * predicted
  q <- as.numeric(mask)
  baseline <- if (statistic == "mean") {
    sum(reference * p) / sum(reference * q)
  } else {
    sum(reference * p)
  }
  if (isTRUE(all.equal(gamma, 1))) {
    return(structure(
      list(lower = baseline, estimate = baseline, upper = baseline,
           gamma = gamma, statistic = statistic,
           domain = domain, domain_label = .inter_domain_label(domain)),
      class = "inter_bounds"
    ))
  }

  system <- .inter_constraint_system(grid, object$margins)
  A <- system$A
  b <- system$b
  nonempty <- rowSums(abs(A)) > 0 | abs(b) > 0
  A <- A[nonempty, , drop = FALSE]
  b <- b[nonempty]
  cells <- nrow(grid)

  if (statistic == "mean") {
    equality <- rbind(cbind(A, -b), c(q, 0))
    directions <- rep.int("=", nrow(equality))
    right_hand_side <- c(rep.int(0, nrow(A)), 1)
    constraints <- equality
    if (is.finite(gamma)) {
      upper <- reference * gamma
      lower <- reference / gamma
      upper_constraints <- cbind(diag(cells), -upper)
      lower_constraints <- cbind(-diag(cells), lower)
      constraints <- rbind(constraints, upper_constraints, lower_constraints)
      directions <- c(directions, rep.int("<=", 2L * cells))
      right_hand_side <- c(right_hand_side, rep.int(0, 2L * cells))
    }
    objective <- c(p, 0)
  } else {
    constraints <- A
    directions <- rep.int("=", nrow(A))
    right_hand_side <- b
    if (is.finite(gamma)) {
      upper <- reference * gamma
      lower <- reference / gamma
      constraints <- rbind(constraints, diag(cells), -diag(cells))
      directions <- c(directions, rep.int("<=", cells), rep.int("<=", cells))
      right_hand_side <- c(right_hand_side, upper, -lower)
    }
    objective <- p
  }
  lower_bound <- .inter_lp("min", objective, constraints, directions,
                           right_hand_side)
  upper_bound <- .inter_lp("max", objective, constraints, directions,
                           right_hand_side)

  structure(
    list(
      lower = lower_bound,
      estimate = baseline,
      upper = upper_bound,
      gamma = gamma,
      statistic = statistic,
      domain = domain,
      domain_label = .inter_domain_label(domain)
    ),
    class = "inter_bounds"
  )
}

#' @export
print.inter_bounds <- function(x, ...) {
  cat("Identification bounds\n")
  cat("  Domain: ", x$domain_label, "\n", sep = "")
  cat("  Statistic: ", x$statistic, "\n", sep = "")
  cat("  Gamma: ", format(x$gamma), "\n", sep = "")
  cat("  Lower / estimate / upper: ",
      paste(format(c(x$lower, x$estimate, x$upper), digits = 6),
            collapse = " / "), "\n", sep = "")
  invisible(x)
}

#' Evaluate association sensitivity across gamma values
#'
#' @param object A fitted `inter_sae` object.
#' @param gamma Numeric vector of sensitivity values.
#' @param domain Optional named list defining one target domain.
#' @param statistic Either `mean` or `total`.
#' @return A data frame of sensitivity bounds.
#' @export
inter_sensitivity <- function(object, gamma = c(1, 1.25, 1.5, 2, Inf),
                              domain = NULL,
                              statistic = c("mean", "total")) {
  statistic <- match.arg(statistic)
  if (!is.numeric(gamma) || !length(gamma) || any(is.na(gamma)) ||
      any(gamma < 1)) {
    stop("Every `gamma` value must be at least 1.", call. = FALSE)
  }
  values <- lapply(gamma, function(current) {
    identification_bounds(object, domain = domain, gamma = current,
                          statistic = statistic)
  })
  data.frame(
    gamma = gamma,
    lower = vapply(values, `[[`, numeric(1), "lower"),
    estimate = vapply(values, `[[`, numeric(1), "estimate"),
    upper = vapply(values, `[[`, numeric(1), "upper"),
    domain = .inter_domain_label(domain),
    statistic = statistic,
    stringsAsFactors = FALSE
  )
}

#' Benchmark domain estimates to an official aggregate
#'
#' @param estimates A fitted `inter_sae` object or a data frame containing
#'   `estimate` and the selected weight column.
#' @param target Official weighted mean/proportion to reproduce.
#' @param weight Name of the benchmarking-weight column.
#' @param method Ratio, additive, or bounded logit benchmarking.
#' @param lower,upper Permitted estimate range for logit benchmarking.
#' @return The input estimates with `original_estimate`, `benchmarked`, and
#'   `benchmark_adjustment` columns.
#' @export
benchmark_inter <- function(estimates, target, weight = "population",
                            method = c("logit", "ratio", "additive"),
                            lower = 0, upper = 1) {
  if (inherits(estimates, "inter_sae")) estimates <- estimates$estimates
  .inter_assert_data_frame(estimates, "estimates")
  method <- match.arg(method)
  if (!all(c("estimate", weight) %in% names(estimates))) {
    stop("The estimate and benchmarking-weight columns were not found.",
         call. = FALSE)
  }
  if (!is.numeric(target) || length(target) != 1L || !is.finite(target)) {
    stop("`target` must be one finite number.", call. = FALSE)
  }
  estimate <- estimates$estimate
  weights <- estimates[[weight]]
  if (any(!is.finite(estimate)) || any(!is.finite(weights)) ||
      any(weights < 0) || sum(weights) <= 0) {
    stop("Estimates and benchmarking weights must be finite; weights must be non-negative with positive total.",
         call. = FALSE)
  }
  current <- .inter_weighted_mean(estimate, weights)
  benchmarked <- switch(
    method,
    ratio = {
      if (abs(current) < .Machine$double.eps) {
        stop("Ratio benchmarking is undefined for a zero current aggregate.",
             call. = FALSE)
      }
      estimate * target / current
    },
    additive = estimate + target - current,
    logit = {
      if (!is.finite(lower) || !is.finite(upper) || lower >= upper ||
          target <= lower || target >= upper) {
        stop("For logit benchmarking, `lower < target < upper` is required.",
             call. = FALSE)
      }
      scaled <- (estimate - lower) / (upper - lower)
      scaled <- pmin(1 - 1e-10, pmax(1e-10, scaled))
      eta <- stats::qlogis(scaled)
      objective <- function(delta) {
        adjusted <- lower + (upper - lower) * stats::plogis(eta + delta)
        .inter_weighted_mean(adjusted, weights) - target
      }
      root <- stats::uniroot(objective, interval = c(-50, 50),
                             tol = 1e-12)$root
      lower + (upper - lower) * stats::plogis(eta + root)
    }
  )
  estimates$original_estimate <- estimate
  estimates$benchmarked <- benchmarked
  estimates$benchmark_adjustment <- benchmarked - estimate
  attr(estimates, "benchmark_target") <- target
  attr(estimates, "benchmark_achieved") <-
    .inter_weighted_mean(benchmarked, weights)
  attr(estimates, "benchmark_method") <- method
  estimates
}

.inter_multiplier_weights <- function(design) {
  data <- design$data
  original <- data[[design$weights]]
  if (!is.null(design$psu)) {
    cluster_variables <- c(design$strata, design$psu)
    cluster_variables <- cluster_variables[!is.null(cluster_variables)]
    cluster_key <- .inter_key(data, cluster_variables)
    cluster_levels <- unique(cluster_key)
    multiplier_by_cluster <- stats::setNames(
      stats::rexp(length(cluster_levels)), cluster_levels
    )
    multiplier <- as.numeric(multiplier_by_cluster[cluster_key])
  } else {
    multiplier <- stats::rexp(nrow(data))
  }
  if (is.null(design$strata)) {
    multiplier <- multiplier * sum(original) /
      sum(original * multiplier)
  } else {
    strata_key <- as.character(data[[design$strata]])
    for (stratum in unique(strata_key)) {
      index <- strata_key == stratum
      denominator <- sum(original[index] * multiplier[index])
      if (denominator > 0) {
        multiplier[index] <- multiplier[index] *
          sum(original[index]) / denominator
      }
    }
  }
  original * multiplier
}

.inter_refit <- function(object, weights) {
  data <- object$design$data
  data$.inter_boot_weight <- weights
  design <- inter_design(
    data,
    weights = ".inter_boot_weight",
    area = object$design$area,
    strata = object$design$strata,
    psu = object$design$psu
  )
  fit_inter_sae(
    object$formula,
    design = design,
    margins = object$margins,
    family = object$family,
    domain = object$domain,
    prior = object$control$prior,
    max_iter = object$control$max_iter,
    tol = object$control$tol,
    residual_correction = object$control$residual_correction,
    max_cells = object$control$max_cells,
    model_args = object$control$model_args
  )
}

#' Bootstrap intersectional estimates
#'
#' @param object A fitted `inter_sae` object.
#' @param R Number of bootstrap replicates.
#' @param type `"multiplier"` or `"replicate"`. The latter uses columns named
#'   in `inter_design(..., replicate_weights=)`.
#' @param seed Random seed for multiplier bootstrap.
#' @param level Confidence level.
#' @param progress Print progress every ten replicates.
#' @return An object of class `inter_bootstrap`.
#' @export
inter_bootstrap <- function(object, R = 199L,
                            type = c("multiplier", "replicate"),
                            seed = 123, level = 0.95, progress = FALSE) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  type <- match.arg(type)
  if (!is.numeric(R) || length(R) != 1L || R < 2) {
    stop("`R` must be an integer of at least 2.", call. = FALSE)
  }
  R <- as.integer(R)
  if (!is.numeric(level) || length(level) != 1L || level <= 0 || level >= 1) {
    stop("`level` must lie strictly between 0 and 1.", call. = FALSE)
  }
  replicate_names <- object$design$replicate_weights
  if (type == "replicate") {
    if (is.null(replicate_names)) {
      stop("No replicate-weight columns were supplied to `inter_design()`.",
           call. = FALSE)
    }
    if (length(replicate_names) < 2L) {
      stop("At least two replicate-weight columns are required.",
           call. = FALSE)
    }
    R <- min(R, length(replicate_names))
  }

  old_seed <- if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
    get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  } else {
    NULL
  }
  on.exit(.inter_restore_rng(old_seed), add = TRUE)
  if (type == "multiplier") set.seed(as.integer(seed))

  base <- object$estimates
  replicate_values <- matrix(NA_real_, nrow = nrow(base), ncol = R)
  failures <- character(0)
  for (r in seq_len(R)) {
    current_weights <- if (type == "multiplier") {
      .inter_multiplier_weights(object$design)
    } else {
      object$design$data[[replicate_names[r]]]
    }
    current <- try(.inter_refit(object, current_weights), silent = TRUE)
    if (inherits(current, "try-error")) {
      failures <- c(failures, as.character(current))
    } else {
      current_key <- .inter_key(current$estimates, object$domain)
      base_key <- .inter_key(base, object$domain)
      replicate_values[, r] <-
        current$estimates$estimate[match(base_key, current_key)]
    }
    if (isTRUE(progress) && (r %% 10L == 0L || r == R)) {
      message("Completed ", r, " of ", R, " bootstrap replicates")
    }
  }
  alpha <- (1 - level) / 2
  summary <- base[c(object$domain, "estimate")]
  summary$bootstrap_se <- apply(replicate_values, 1L, stats::sd, na.rm = TRUE)
  summary$lower <- apply(replicate_values, 1L, function(x) {
    if (sum(is.finite(x)) < 2L) return(NA_real_)
    as.numeric(stats::quantile(x, alpha, na.rm = TRUE, names = FALSE,
                               type = 8))
  })
  summary$upper <- apply(replicate_values, 1L, function(x) {
    if (sum(is.finite(x)) < 2L) return(NA_real_)
    as.numeric(stats::quantile(x, 1 - alpha, na.rm = TRUE, names = FALSE,
                               type = 8))
  })
  summary$successful_replicates <- rowSums(is.finite(replicate_values))

  structure(
    list(
      summary = summary,
      replicates = replicate_values,
      type = type,
      requested = R,
      level = level,
      failures = failures
    ),
    class = "inter_bootstrap"
  )
}

#' @export
print.inter_bootstrap <- function(x, ...) {
  cat("interSAE bootstrap inference\n")
  cat("  Type: ", x$type, "\n", sep = "")
  cat("  Requested replicates: ", x$requested, "\n", sep = "")
  cat("  Completely failed fits: ", length(x$failures), "\n", sep = "")
  print(x$summary, row.names = FALSE)
  invisible(x)
}

#' Summarize available uncertainty components
#'
#' @param object A fitted `inter_sae` object.
#' @param bootstrap Optional result from [inter_bootstrap()].
#' @param bounds Optional single-domain result from
#'   [identification_bounds()].
#' @return A data frame. Bootstrap refitting combines sampling and fitted-model
#'   variation; identification width is reported separately.
#' @export
decompose_uncertainty <- function(object, bootstrap = NULL, bounds = NULL) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  out <- object$estimates[c(object$domain, "estimate")]
  out$sampling_model_se <- NA_real_
  out$bootstrap_lower <- NA_real_
  out$bootstrap_upper <- NA_real_
  if (!is.null(bootstrap)) {
    if (!inherits(bootstrap, "inter_bootstrap")) {
      stop("`bootstrap` must be returned by `inter_bootstrap()`.",
           call. = FALSE)
    }
    out$sampling_model_se <- bootstrap$summary$bootstrap_se
    out$bootstrap_lower <- bootstrap$summary$lower
    out$bootstrap_upper <- bootstrap$summary$upper
  }
  out$identification_lower <- NA_real_
  out$identification_upper <- NA_real_
  out$identification_width <- NA_real_
  if (!is.null(bounds)) {
    if (!inherits(bounds, "inter_bounds")) {
      stop("`bounds` must be returned by `identification_bounds()`.",
           call. = FALSE)
    }
    selected <- .inter_match_rows(out, bounds$domain)
    if (sum(selected) != 1L) {
      stop("The bounds domain must identify exactly one output row.",
           call. = FALSE)
    }
    out$identification_lower[selected] <- bounds$lower
    out$identification_upper[selected] <- bounds$upper
    out$identification_width[selected] <- bounds$upper - bounds$lower
  }
  out
}

.inter_prepare_model_data <- function(formula, survey, grid) {
  variables <- all.vars(formula[[3L]])
  missing_grid <- setdiff(variables, names(grid))
  if (length(missing_grid)) {
    stop("Every model predictor must be represented in the census margins. Missing: ",
         paste(missing_grid, collapse = ", "), call. = FALSE)
  }
  survey_model <- survey
  grid_model <- grid
  for (variable in variables) {
    survey_values <- survey[[variable]]
    grid_values <- grid[[variable]]
    if (is.numeric(survey_values) || is.integer(survey_values)) {
      converted <- suppressWarnings(as.numeric(as.character(grid_values)))
      if (anyNA(converted)) {
        stop("Numeric survey predictor `", variable,
             "` has non-numeric census categories.", call. = FALSE)
      }
      grid_model[[variable]] <- converted
    } else {
      levels_all <- unique(c(as.character(survey_values),
                             as.character(grid_values)))
      observed <- unique(as.character(survey_values[!is.na(survey_values)]))
      census_only <- setdiff(unique(as.character(grid_values)), observed)
      if (length(census_only)) {
        stop("Predictor `", variable,
             "` contains census levels absent from the entire survey: ",
             paste(census_only, collapse = ", "), call. = FALSE)
      }
      survey_model[[variable]] <- factor(as.character(survey_values),
                                         levels = levels_all)
      grid_model[[variable]] <- factor(as.character(grid_values),
                                       levels = levels_all)
    }
  }
  list(survey = survey_model, grid = grid_model, variables = variables)
}

.inter_calibrated_weights <- function(design, population) {
  dims <- setdiff(names(population$grid),
                  c(".cell", ".population", ".seed", ".prediction"))
  grid_key <- .inter_key(population$grid, dims)
  survey_key <- .inter_key(design$data, dims)
  cell <- match(survey_key, grid_key)
  original <- design$data[[design$weights]]
  sample_total <- numeric(nrow(population$grid))
  by_cell <- tapply(original[!is.na(cell)], cell[!is.na(cell)], sum,
                    na.rm = TRUE)
  if (length(by_cell)) {
    sample_total[as.integer(names(by_cell))] <- as.numeric(by_cell)
  }
  factor <- numeric(nrow(population$grid))
  supported <- sample_total > 0
  factor[supported] <- population$grid$.population[supported] /
    sample_total[supported]
  adjusted <- rep.int(0, length(original))
  adjusted[!is.na(cell)] <- original[!is.na(cell)] * factor[cell[!is.na(cell)]]
  adjusted
}

.inter_domain_estimates <- function(grid, survey, original_weights,
                                    calibrated_weights, outcome,
                                    prediction_survey, domain,
                                    residual_correction, family_label) {
  cell_key <- .inter_key(grid, domain)
  domain_keys <- unique(cell_key)
  representative <- grid[match(domain_keys, cell_key), domain, drop = FALSE]
  rownames(representative) <- NULL

  population <- as.numeric(.inter_group_sum(grid$.population, cell_key,
                                            domain_keys))
  synthetic_total <- as.numeric(.inter_group_sum(
    grid$.population * grid$.prediction, cell_key, domain_keys
  ))
  synthetic <- synthetic_total / population

  survey_key <- .inter_key(survey, domain)
  y <- survey[[outcome]]
  valid_model <- is.finite(y) & is.finite(prediction_survey) &
    is.finite(calibrated_weights)
  correction_total <- rep.int(0, length(domain_keys))
  if (isTRUE(residual_correction) && any(valid_model)) {
    correction_total <- as.numeric(.inter_group_sum(
      calibrated_weights[valid_model] *
        (y[valid_model] - prediction_survey[valid_model]),
      survey_key[valid_model], domain_keys
    ))
  }
  total <- synthetic_total + correction_total
  estimate <- total / population
  if (identical(family_label, "binomial")) {
    estimate <- pmin(1, pmax(0, estimate))
    total <- estimate * population
  }

  valid_direct <- is.finite(y) & is.finite(original_weights) &
    original_weights > 0
  direct_n <- as.numeric(table(factor(survey_key[valid_direct],
                                      levels = domain_keys)))
  direct_weight <- as.numeric(.inter_group_sum(
    original_weights[valid_direct], survey_key[valid_direct], domain_keys
  ))
  direct_total <- as.numeric(.inter_group_sum(
    original_weights[valid_direct] * y[valid_direct],
    survey_key[valid_direct], domain_keys
  ))
  direct <- ifelse(direct_weight > 0, direct_total / direct_weight, NA_real_)

  result <- cbind(
    representative,
    data.frame(
      estimate = estimate,
      total = total,
      population = population,
      synthetic = synthetic,
      residual_correction = correction_total / population,
      direct = direct,
      direct_n = direct_n,
      stringsAsFactors = FALSE
    )
  )
  result[is.finite(result$population) & result$population > 0, , drop = FALSE]
}

#' Fit joint margin-constrained intersectional estimates
#'
#' @param formula Model formula. The response is observed in the survey and
#'   every predictor must occur among the census-margin dimensions.
#' @param design An object returned by [inter_design()].
#' @param margins An object returned by [census_margins()].
#' @param family Either `"gaussian"`, `"binomial"`, or an R family object.
#' @param domain Character vector defining output domains. Defaults to area.
#' @param prior Positive seed added to every feasible latent cell.
#' @param max_iter Maximum generalized-IPF iterations.
#' @param tol Relative margin tolerance.
#' @param residual_correction Add a calibrated survey-residual correction.
#' @param max_cells Safety limit on the latent joint table.
#' @param model_args Optional named list passed to [stats::glm()].
#' @return An object of class `inter_sae`.
#' @export
fit_inter_sae <- function(formula, design, margins,
                          family = c("gaussian", "binomial"),
                          domain = NULL, prior = 0.5,
                          max_iter = 1000L, tol = 1e-8,
                          residual_correction = TRUE,
                          max_cells = 1e6, model_args = list()) {
  if (!inherits(design, "inter_design")) {
    stop("`design` must be created by `inter_design()`.", call. = FALSE)
  }
  if (!inherits(margins, "census_margins")) {
    stop("`margins` must be created by `census_margins()`.", call. = FALSE)
  }
  if (!inherits(formula, "formula") || length(formula) != 3L ||
      !is.symbol(formula[[2L]])) {
    stop("`formula` must contain one response and predictors.", call. = FALSE)
  }
  outcome <- all.vars(formula[[2L]])
  if (length(outcome) != 1L || !outcome %in% names(design$data)) {
    stop("The formula response must be one survey-data column.", call. = FALSE)
  }
  if (is.null(domain)) domain <- design$area
  if (!is.character(domain) || !length(domain) ||
      !all(domain %in% margins$dimensions)) {
    stop("`domain` must name one or more census dimensions.", call. = FALSE)
  }
  domain <- unique(domain)
  if (!is.numeric(max_iter) || length(max_iter) != 1L || max_iter < 1 ||
      !is.numeric(tol) || length(tol) != 1L || !is.finite(tol) || tol <= 0 ||
      !is.numeric(max_cells) || length(max_cells) != 1L || max_cells < 1) {
    stop("`max_iter` and `max_cells` must be positive; `tol` must be positive and finite.",
         call. = FALSE)
  }
  if (!is.list(model_args) ||
      any(names(model_args) %in% c("formula", "family", "data", "weights",
                                   "na.action", "model"))) {
    stop("`model_args` must be a list and may not override core model arguments.",
         call. = FALSE)
  }
  family_object <- if (missing(family)) {
    stats::gaussian()
  } else if (is.character(family)) {
    .inter_family(match.arg(family))
  } else {
    .inter_family(family)
  }
  family_label <- if (family_object$family %in% c("binomial", "quasibinomial")) {
    "binomial"
  } else {
    "gaussian"
  }
  y <- design$data[[outcome]]
  if (family_label == "binomial" &&
      any(!is.na(y) & !(y %in% c(0, 1)))) {
    stop("A binomial response must contain only 0, 1, or missing values.",
         call. = FALSE)
  }

  population <- .inter_reconstruct(
    design, margins, prior = prior, max_iter = as.integer(max_iter),
    tol = tol, max_cells = max_cells
  )
  prepared <- .inter_prepare_model_data(formula, design$data,
                                        population$grid)
  fitting_family <- if (family_label == "binomial") {
    stats::quasibinomial(link = family_object$link)
  } else {
    family_object
  }
  glm_call <- c(
    list(
      formula = formula,
      family = fitting_family,
      data = prepared$survey,
      weights = prepared$survey[[design$weights]],
      na.action = stats::na.exclude,
      model = TRUE
    ),
    model_args
  )
  model <- do.call(stats::glm, glm_call)
  prediction_cell <- as.numeric(stats::predict(
    model, newdata = prepared$grid, type = "response"
  ))
  if (any(!is.finite(prediction_cell))) {
    stop(paste0(
      "The outcome model cannot predict every census cell. ",
      "Simplify aliased interactions or collapse unsupported categories."
    ),
         call. = FALSE)
  }
  prediction_survey <- as.numeric(stats::predict(
    model, newdata = prepared$survey, type = "response"
  ))
  population$grid$.prediction <- prediction_cell
  calibrated_weights <- .inter_calibrated_weights(design, population)
  estimates <- .inter_domain_estimates(
    population$grid,
    prepared$survey,
    design$data[[design$weights]],
    calibrated_weights,
    outcome,
    prediction_survey,
    domain,
    residual_correction,
    family_label
  )

  structure(
    list(
      call = match.call(),
      formula = formula,
      outcome = outcome,
      family = family_object,
      family_label = family_label,
      domain = domain,
      design = design,
      margins = margins,
      population = population,
      model = model,
      estimates = estimates,
      calibrated_weights = calibrated_weights,
      control = list(
        prior = prior,
        max_iter = max_iter,
        tol = tol,
        residual_correction = residual_correction,
        max_cells = max_cells,
        model_args = model_args
      )
    ),
    class = "inter_sae"
  )
}

#' Extract intersectional estimates
#'
#' @param object A fitted `inter_sae` object.
#' @param statistic Return domain means/proportions, totals, or both.
#' @return A data frame.
#' @export
inter_estimates <- function(object, statistic = c("mean", "total", "both")) {
  if (!inherits(object, "inter_sae")) {
    stop("`object` must be a fitted `inter_sae` object.", call. = FALSE)
  }
  statistic <- match.arg(statistic)
  keep <- c(object$domain, "population", "direct", "direct_n")
  if (statistic %in% c("mean", "both")) {
    keep <- c(object$domain, "estimate", "synthetic", "residual_correction",
              "population", "direct", "direct_n")
  }
  if (statistic == "total") {
    keep <- c(object$domain, "total", "population", "direct_n")
  }
  if (statistic == "both") {
    keep <- c(object$domain, "estimate", "total", "synthetic",
              "residual_correction", "population", "direct", "direct_n")
  }
  object$estimates[keep]
}

test_that("survey and census inputs are validated", {
  x <- inter_example()
  design <- inter_design(x$survey, "weight", "area", "strata", "psu")
  margins <- census_margins(x$margins)
  expect_s3_class(design, "inter_design")
  expect_s3_class(margins, "census_margins")
  expect_equal(length(margins$tables), 3L)
  expect_error(inter_design(x$survey, "missing", "area"), "not found")
})

test_that("fit reproduces margins and finite bounded estimates", {
  x <- inter_example()
  design <- inter_design(x$survey, "weight", "area", "strata", "psu")
  margins <- census_margins(x$margins)
  fit <- fit_inter_sae(
    unemployed ~ sex + age + residence + area,
    design, margins,
    family = "binomial",
    domain = c("area", "sex")
  )
  expect_s3_class(fit, "inter_sae")
  estimates <- inter_estimates(fit)
  expect_true(all(is.finite(estimates$estimate)))
  expect_true(all(estimates$estimate >= 0 & estimates$estimate <= 1))
  expect_lt(max(margin_diagnostics(fit)$relative_error), 1e-6)
  expect_equal(sum(estimates$population), margins$total)
})

test_that("rank diagnostic recognizes partial identification", {
  x <- inter_example()
  fit <- fit_inter_sae(
    unemployed ~ sex + age + residence + area,
    inter_design(x$survey, "weight", "area"),
    census_margins(x$margins),
    family = "binomial"
  )
  diagnostic <- check_identifiability(fit)
  expect_s3_class(diagnostic, "inter_identifiability")
  expect_gte(diagnostic$nullity, 0)
})

test_that("benchmarking reaches the requested aggregate", {
  x <- inter_example()
  fit <- fit_inter_sae(
    unemployed ~ sex + age + residence + area,
    inter_design(x$survey, "weight", "area"),
    census_margins(x$margins),
    family = "binomial"
  )
  target <- 0.3
  adjusted <- benchmark_inter(fit, target)
  expect_equal(attr(adjusted, "benchmark_achieved"), target, tolerance = 1e-8)
  expect_true(all(adjusted$benchmarked > 0 & adjusted$benchmarked < 1))
})

# Statistical method implemented in interSAE 0.1.0

Let `s` denote a probability sample with outcome `y`, design weight `d`, area
`a`, and categorical auxiliary vector `x`. Let `N` be the unknown vector of
counts in the complete area-by-auxiliary population table. Published census
margins are represented as

```text
A N = b,   N >= 0,
```

where each row of `A` aggregates complete cells to one published census cell.
Structural-zero declarations remove impossible complete cells before fitting.

## Population reconstruction

The weighted survey table, after adding a positive support prior, is used as a
seed. Generalized iterative proportional fitting computes the minimum-divergence
projection of the seed onto all supplied margin constraints. This fitted table
is not described as identified when the rank of `A` is smaller than the number
of feasible complete cells.

## Outcome estimator

A design-weighted generalized linear model estimates the cell prediction
`m_hat(x)`. For a reported domain `D`, the synthetic total is

```text
T_syn(D) = sum_{c in D} N_hat(c) m_hat(c).
```

Survey weights are calibrated within supported complete cells to their fitted
population counts. The model-assisted total adds a residual term:

```text
T_MA(D) = T_syn(D) + sum_{i in s and D} w_cal(i) [y(i) - m_hat(x(i))].
```

The domain mean or proportion is `T_MA(D) / N_hat(D)`. For a binary response,
the final proportion is restricted to the unit interval.

## Identification sensitivity

The fitted joint population table is one member of a generally non-singleton
feasible set. With cell prediction vector `p` and domain indicator `q`, a
synthetic domain mean is the linear-fractional target

```text
(p' N) / (q' N).
```

`identification_bounds()` applies the Charnes--Cooper transformation and linear
programming to minimize and maximize this target subject to the census
constraints. A finite sensitivity value `gamma` adds

```text
N_hat(c) / gamma <= N(c) <= gamma N_hat(c).
```

`gamma = Inf` removes these association restrictions. Bounds are conditional
on fitted cell predictions and concern latent-table non-identification; they do
not replace sampling inference.

## Sampling and refitting variation

The multiplier bootstrap draws one exponential multiplier per PSU, then
renormalizes weights within strata. Each replicate repeats population
reconstruction and outcome-model fitting. Supplied replicate-weight columns
can be used instead. The resulting standard error combines survey-weight and
model-refitting variation conditional on fixed published census margins.

## Required reporting

Applications should report the survey design, outcome formula, supplied
margins, structural zeros, reconstruction tolerance, nullity, bootstrap method,
and all sensitivity values. A narrow bootstrap interval does not resolve a wide
identification interval.

---
title: "Joint Margin-Constrained Intersectional Estimation"
author: "Leila Marvian Mashhad"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Joint Margin-Constrained Intersectional Estimation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Problem

A probability survey observes an outcome and a rich set of person-level
attributes. Census dissemination supplies reliable aggregate tables, but the
full cross-classification is unavailable. `interSAE` treats the unobserved
multiway census table as a latent population constrained by every published
margin.

## Fit

```{r}
library(interSAE)
x <- inter_example()
design <- inter_design(x$survey, "weight", "area", "strata", "psu")
margins <- census_margins(x$margins)
fit <- fit_inter_sae(
  unemployed ~ sex + age + residence + area,
  design, margins,
  family = "binomial",
  domain = c("area", "sex")
)
head(inter_estimates(fit))
```

The `synthetic` column is the census-cell prediction aggregated with the
reconstructed population counts. The residual correction is a calibrated
survey residual total divided by domain population. Their sum gives the
model-assisted estimate.

## Coherence and identification

```{r}
max(margin_diagnostics(fit)$relative_error)
check_identifiability(fit)
```

Nullity greater than zero means that overlapping margins do not uniquely
identify the full latent table. It is not an algorithmic failure. It is a
property of the available information.

When `lpSolve` is installed, association sensitivity can be evaluated by
restricting each latent cell to a multiplicative neighborhood of its fitted
value while maintaining all census margins exactly.

```{r, eval=requireNamespace("lpSolve", quietly=TRUE)}
inter_sensitivity(
  fit,
  gamma = c(1, 1.25, 1.5, 2),
  domain = list(area = "A1", sex = "Female")
)
```

## Bootstrap inference

The multiplier bootstrap assigns one exponential multiplier per PSU and
renormalizes within strata. Every replicate repeats both population
reconstruction and outcome-model fitting.

```{r, eval=FALSE}
boot <- inter_bootstrap(fit, R = 199, type = "multiplier", seed = 42)
boot$summary
```

Bootstrap variation and identification width should not be silently collapsed
into one standard error. `decompose_uncertainty()` reports them separately.

\name{fit_inter_sae}
\alias{fit_inter_sae}
\alias{inter_estimates}
\alias{margin_diagnostics}
\alias{print.inter_sae}
\alias{summary.inter_sae}
\alias{print.summary_inter_sae}
\alias{plot.inter_sae}
\title{Fit and Inspect Intersectional Small Area Estimates}
\usage{
fit_inter_sae(formula, design, margins,
  family = c("gaussian", "binomial"), domain = NULL, prior = 0.5,
  max_iter = 1000L, tol = 1e-8, residual_correction = TRUE,
  max_cells = 1e6, model_args = list())

inter_estimates(object, statistic = c("mean", "total", "both"))

margin_diagnostics(object)

\method{print}{inter_sae}(x, ...)

\method{summary}{inter_sae}(object, ...)

\method{print}{summary_inter_sae}(x, ...)

\method{plot}{inter_sae}(x, y = NULL, compare_direct = TRUE, ...)
}
\arguments{
\item{formula}{Outcome model formula. Every predictor must occur among the
census-margin dimensions.}
\item{design}{An object returned by \code{inter_design()}.}
\item{margins}{An object returned by \code{census_margins()}.}
\item{family}{Gaussian or binomial response family.}
\item{domain}{Character vector of output-domain dimensions.}
\item{prior}{Positive seed added to every feasible latent population cell.}
\item{max_iter}{Maximum generalized iterative-proportional-fitting iterations.}
\item{tol}{Relative convergence tolerance.}
\item{residual_correction}{Add a calibrated survey-residual correction.}
\item{max_cells}{Maximum permitted latent-table size.}
\item{model_args}{Optional arguments passed to \code{stats::glm()}.}
\item{object,x}{A fitted \code{inter_sae} object.}
\item{statistic}{Extract means/proportions, totals, or both.}
\item{y}{Ignored.}
\item{compare_direct}{Overlay direct survey estimates in the plot.}
\item{...}{Additional printing or plotting arguments.}
}
\value{
\code{fit_inter_sae()} returns an \code{inter_sae} object.
\code{inter_estimates()} and \code{margin_diagnostics()} return data frames.
The summary method returns a \code{summary_inter_sae} object. The plot method
returns the plotted data invisibly.
}
\description{
Reconstruct a coherent latent joint population table, fit a survey-weighted
outcome model, and calculate augmented model-assisted estimates.
}
\details{
For binomial outcomes the working model uses a quasibinomial likelihood so
non-integer survey weights are interpreted as analytic weights. Returned
estimates remain proportions on the response scale.
}
\examples{
example_data <- inter_example()
design <- inter_design(example_data$survey, "weight", "area",
  "strata", "psu")
margins <- census_margins(example_data$margins)
fit <- fit_inter_sae(unemployed ~ sex + age + residence + area,
  design, margins, family = "binomial", domain = c("area", "sex"))
head(inter_estimates(fit))
max(margin_diagnostics(fit)$relative_error)
summary(fit)
}

\name{simulate_inter_data}
\alias{simulate_inter_data}
\alias{inter_example}
\title{Simulate an Intersectional Survey--Census Problem}
\usage{
simulate_inter_data(seed = 123, n = 500, areas = paste0("A", 1:6))

inter_example()
}
\arguments{
\item{seed}{Random-number seed.}
\item{n}{Survey sample size.}
\item{areas}{Distinct area labels.}
}
\value{
A list containing survey microdata, overlapping census margins, and the latent
full population table used to generate the data.
}
\description{
Create a reproducible problem with a binary unemployment outcome and partially
identified age-by-residence associations within area and sex groups.
}
\examples{
example_data <- inter_example()
names(example_data)
head(example_data$survey)
lapply(example_data$margins, head)
}

\name{interSAE-package}
\alias{interSAE}
\alias{interSAE-package}
\docType{package}
\title{Intersectional Small Area Estimation from Survey and Census Data}
\description{
The \pkg{interSAE} package combines complex-survey microdata with overlapping
aggregate census margins. It reconstructs a coherent latent population table
and produces model-assisted estimates for geographic areas crossed with
demographic subgroups.
}
\details{
The package separates survey-design declaration, census-margin validation,
constrained population reconstruction, and outcome estimation. Optional tools
diagnose non-identification, calculate association-sensitivity bounds,
bootstrap the complete fitting procedure, and benchmark estimates.
}
\references{
Deville, J.-C. and Sarndal, C.-E. (1992). Calibration estimators in survey
sampling. \emph{Journal of the American Statistical Association}, 87,
376--382. \doi{10.1080/01621459.1992.10475217}.

Fay, R. E. and Herriot, R. A. (1979). Estimates of income for small places.
\emph{Journal of the American Statistical Association}, 74, 269--277.
\doi{10.1080/01621459.1979.10482505}.
}
\author{Leila Marvian Mashhad}
\keyword{package}
\keyword{survey}

\name{inter_bootstrap}
\alias{inter_bootstrap}
\alias{print.inter_bootstrap}
\alias{decompose_uncertainty}
\title{Bootstrap and Summarize Uncertainty}
\usage{
inter_bootstrap(object, R = 199L,
  type = c("multiplier", "replicate"), seed = 123,
  level = 0.95, progress = FALSE)

decompose_uncertainty(object, bootstrap = NULL, bounds = NULL)

\method{print}{inter_bootstrap}(x, ...)
}
\arguments{
\item{object}{A fitted \code{inter_sae} object.}
\item{R}{Number of bootstrap replicates.}
\item{type}{Multiplier or supplied-replicate-weight bootstrap.}
\item{seed}{Random seed for multiplier bootstrap.}
\item{level}{Confidence level.}
\item{progress}{Print periodic progress messages.}
\item{bootstrap}{Optional result from \code{inter_bootstrap()}.}
\item{bounds}{Optional result from \code{identification_bounds()}.}
\item{x}{Object to print.}
\item{...}{Unused.}
}
\value{
\code{inter_bootstrap()} returns an \code{inter_bootstrap} object containing a
summary and replicate matrix. \code{decompose_uncertainty()} returns a data
frame.
}
\description{
Refit population reconstruction and the outcome model under each set of
weights. Standard errors include sampling and refitting variation.
Identification width can be reported separately.
}
\examples{
\dontrun{
example_data <- inter_example()
design <- inter_design(example_data$survey, "weight", "area", "strata", "psu")
margins <- census_margins(example_data$margins)
fit <- fit_inter_sae(unemployed ~ sex + age + residence + area,
  design, margins, family = "binomial")
boot <- inter_bootstrap(fit, R = 99)
boot
}
}

\name{benchmark_inter}
\alias{benchmark_inter}
\title{Benchmark Domain Estimates to an Official Aggregate}
\usage{
benchmark_inter(estimates, target, weight = "population",
  method = c("logit", "ratio", "additive"), lower = 0, upper = 1)
}
\arguments{
\item{estimates}{A fitted \code{inter_sae} object or estimate data frame.}
\item{target}{Official weighted mean or proportion.}
\item{weight}{Benchmarking-weight column.}
\item{method}{Logit, ratio, or additive adjustment.}
\item{lower,upper}{Range for bounded logit adjustment.}
}
\value{A data frame with original and benchmarked estimates.}
\description{
Apply a common adjustment while reproducing an official population-weighted
aggregate. Logit adjustment preserves a bounded response range.
}
\examples{
example_data <- inter_example()
design <- inter_design(example_data$survey, "weight", "area")
margins <- census_margins(example_data$margins)
fit <- fit_inter_sae(unemployed ~ sex + age + residence + area,
  design, margins, family = "binomial")
official <- with(example_data$survey,
  weighted.mean(unemployed, weight))
benchmarked <- benchmark_inter(fit, official)
attr(benchmarked, "benchmark_achieved")
}

\name{identification_bounds}
\alias{check_identifiability}
\alias{print.inter_identifiability}
\alias{identification_bounds}
\alias{print.inter_bounds}
\alias{inter_sensitivity}
\title{Diagnose Non-Identification and Compute Sensitivity Bounds}
\usage{
check_identifiability(object, tol = 1e-9)

identification_bounds(object, domain = NULL, gamma = Inf,
  statistic = c("mean", "total"))

inter_sensitivity(object, gamma = c(1, 1.25, 1.5, 2, Inf),
  domain = NULL, statistic = c("mean", "total"))

\method{print}{inter_identifiability}(x, ...)

\method{print}{inter_bounds}(x, ...)
}
\arguments{
\item{object}{A fitted \code{inter_sae} object.}
\item{tol}{Numerical-rank tolerance.}
\item{domain}{Optional named list defining one target domain.}
\item{gamma}{Association-sensitivity value or vector. One fixes the fitted
latent table; infinity imposes census margins only.}
\item{statistic}{Bound a predicted domain mean/proportion or total.}
\item{x}{Object to print.}
\item{...}{Unused.}
}
\value{
\code{check_identifiability()} returns an \code{inter_identifiability} object.
\code{identification_bounds()} returns an \code{inter_bounds} object.
\code{inter_sensitivity()} returns a data frame.
}
\description{
Measure the nullity left by overlapping aggregate margins and, when
\pkg{lpSolve} is installed, optimize the target over latent population tables
that satisfy the margins and optional association restrictions.
}
\details{
Mean bounds use a Charnes--Cooper transformation of the linear-fractional
target. They concern uncertainty in the latent census table conditional on the
fitted cell predictions; they are not replacements for sampling intervals.
}
\examples{
example_data <- inter_example()
design <- inter_design(example_data$survey, "weight", "area")
margins <- census_margins(example_data$margins)
fit <- fit_inter_sae(unemployed ~ sex + age + residence + area,
  design, margins, family = "binomial")
check_identifiability(fit)
if (requireNamespace("lpSolve", quietly = TRUE)) {
  identification_bounds(fit, domain = list(area = "A1"), gamma = 1.5)
}
}

\name{inter_design}
\alias{inter_design}
\alias{print.inter_design}
\alias{census_margins}
\alias{print.census_margins}
\title{Declare Survey Design Information and Census Margins}
\usage{
inter_design(data, weights, area, strata = NULL, psu = NULL,
  replicate_weights = NULL)

census_margins(margins, count = "Freq", structural_zeros = NULL,
  tolerance = 1e-8, reconcile = FALSE)

\method{print}{inter_design}(x, ...)

\method{print}{census_margins}(x, ...)
}
\arguments{
\item{data}{Survey microdata.}
\item{weights}{Name of the survey-weight column.}
\item{area}{Name of the geographic-area column.}
\item{strata}{Optional stratum-column name.}
\item{psu}{Optional primary-sampling-unit column name.}
\item{replicate_weights}{Optional character vector naming replicate-weight columns.}
\item{margins}{A census table or named list of census tables.}
\item{count}{Common name of the population-count column.}
\item{structural_zeros}{Optional data frame of impossible category combinations.
Missing values act as wildcards.}
\item{tolerance}{Relative tolerance for equality of census-table totals.}
\item{reconcile}{Whether inconsistent table totals may be proportionally rescaled.}
\item{x}{Object to print.}
\item{...}{Unused.}
}
\value{
\code{inter_design()} returns an object of class \code{inter_design}.
\code{census_margins()} returns an object of class \code{census_margins}.
}
\description{
Validate the two inputs required by \code{fit_inter_sae()}. Column names are
supplied as strings to keep programmatic workflows unambiguous.
}
\examples{
example_data <- inter_example()
design <- inter_design(example_data$survey, weights = "weight",
  area = "area", strata = "strata", psu = "psu")
margins <- census_margins(example_data$margins)
design
margins
}
